@@ -1,6 +1,11 @@
 //! Traits and functions for general purpose, everyday mathematics.
 //! Everything you need.
+
+use std::convert::{TryInto, TryFrom};
+
 use crate::eval_postfix;
+
+use super::constants::{SQRT5, GOLDENRATIO};
 pub mod avg_impl;
 /// Trait for several kinds of averages.
 pub trait Averages<T> {
@@ -83,7 +88,7 @@ pub trait NumDigits {
     /// 
     /// assert_eq!(3u8, 12.cross_sum());
     /// assert_eq!(9u16, 342.cross_sum());
-    /// assert_eq!(52u64, 4928947234.cross_sum());
+    /// assert_eq!(52u64, 4928947234u64.cross_sum());
     /// ```
     #[must_use]
     fn cross_sum(&self) -> Self;
@@ -221,126 +226,55 @@ pub trait Decrement {
 }
 
 impl<T: std::ops::AddAssign + From<u8>> Increment for T {
+    #[inline]
     fn inc(&mut self) {
         *self += 1u8.into();
     }
-
+    #[inline]
     fn inc_by(&mut self, n: Self) {
         *self += n;
     }
 }
 
 impl<T: std::ops::SubAssign + From<u8>> Decrement for T {
+    #[inline]
     fn dec(&mut self) {
         *self -= 1u8.into();
     }
 
+    #[inline]
     fn dec_by(&mut self, n: Self) {
         *self -= n;
     }
 }
 
-impl NumDigits for u8 {
+impl<'a, T: std::cmp::PartialEq +
+        std::ops::DivAssign + std::cmp::PartialOrd +
+        std::ops::Rem<Output = T> + Copy + std::ops::AddAssign>
+    NumDigits
+    for T
+    where
+        u8: TryFrom<T>,
+        T: From<u8> {
     fn cross_sum(&self) -> Self {
-        self.digits().iter().sum()
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u8{ return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(3);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n);
-        }
-        digits
-    }
-}
-
-impl NumDigits for u16 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
+        let d: Vec<u8> = self.digits();
+        let mut res: Self = 0.into();
+        for i in d {
+            res.inc_by(i.into());
         }
         res
     }
 
     fn digits(&self) -> Vec<u8> {
-        if self == &0u16 { return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(5);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
-        }
-        digits
-    }
-}
-
-impl NumDigits for u32 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
-        }
-        res
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u32 { return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(10);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
-        }
-        digits
-    }
-}
-
-impl NumDigits for u64 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
-        }
-        res
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u64 { return vec![0]; }
-        let mut v: Self = *self;
-        let mut digits: Vec<u8> = Vec::with_capacity(20);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
-        }
-        digits
-    }
-}
-
-impl NumDigits for u128 {
-    fn cross_sum(&self) -> Self {
-        let mut res = 0;
-        for i in self.digits() {
-            res.inc_by(i as Self);
-        }
-        res
-    }
-
-    fn digits(&self) -> Vec<u8> {
-        if self == &0u128 { return vec![0]; }
+        if self == &0u8.into() { return vec![0]; }
         let mut v: Self = *self;
         let mut digits: Vec<u8> = Vec::with_capacity(39);
-        while v > 0 {
-            let n: Self = v % 10;
-            v /= 10;
-            digits.push(n as u8);
+        while v > 0.into() {
+            let n: u8 = unsafe { (v % 10.into()).try_into().unwrap_unchecked() };
+            if n == 255
+            { panic!("Oops! Something went wrong. please contact the developers using the error code 0x00001.")}
+            v /= 10.into();
+            digits.push(n);
         }
         digits
     }
@@ -351,16 +285,18 @@ impl<T: std::ops::Add<Output = T> +
         std::ops::Mul<Output = T> + 
         std::ops::Div<Output = T> + 
         Copy> MapToNumRange<T> for T {
+            #[inline]
             fn map_to(&self, start1: T, end1: T, start2: T, end2: T) -> T {
                 eval_postfix!(start2 end2 start2 - + (*self) start1 - end1 / start1 - *)
             }
         }
 
 impl<T: std::ops::Mul<Output = T> + Copy> CommonPowers for T {
+    #[inline]
     fn square(&self) -> Self {
         *self * *self
     }
-
+    #[inline]
     fn cube(&self) -> Self {
         *self * *self * *self
     }
@@ -375,28 +311,47 @@ impl<T: std::ops::Mul<Output = T> + Copy> CommonPowers for T {
 /// ```
 /// use lib_rapid::math::general::nth_root;
 /// use lib_rapid::math::constants;
-/// assert_eq!(1.4422495703074085, nth_root(3.0, 3.0));
+/// assert_eq!(constants::CUBEROOT3, nth_root(3.0, 3.0));
 /// ```
 /// ```
 /// use lib_rapid::math::general::nth_root;
 /// use lib_rapid::math::constants;
-/// assert_eq!(1.414213562373095, nth_root(2.0, 2.0));
+/// assert_eq!(constants::SQRT2, nth_root(2.0, 2.0));
 /// ```
+#[inline]
 pub fn nth_root(degree: f64, radicand: f64) -> f64 {
-    // Using Newtons method
-    let     p:             f64 = 1e-15_f64;
-    let mut initial_guess: f64 = radicand / degree;
-    let mut next_guess:    f64;
-  
-    loop {
-       next_guess = ((degree - 1.0) * initial_guess 
-                    + radicand / 
-                    f64::powf(initial_guess, degree - 1.0))
-                    / degree; // Guess the next guess
-
-       if (next_guess - initial_guess).abs() <= (initial_guess * p).abs()
-       { return next_guess; }
-
-       initial_guess = next_guess;
+    radicand.powf(degree.recip())
+}
+/// Computes th nth fibonacci number (up to 186th) accurately using the fastest available computing method.
+/// # Arguments
+/// * `n` - the nth-fibonacci number to be computed.
+/// # How does it work?
+/// This function uses the function `φⁿ / √5` for all numbers `n < 76`, as this is the maximum precision for that formula. For every number above that, it uses the iterative approach.
+/// # Examples
+/// ```
+/// use lib_rapid::math::general::nth_fibonacci;
+/// 
+/// assert_eq!(1304969544928657, nth_fibonacci(74));
+/// assert_eq!(332825110087067562321196029789634457848, nth_fibonacci(186));
+/// ```
+pub fn nth_fibonacci(n: u128) -> u128 {
+    if n > 186
+    { panic!("Error: The 187th Fibonacci number and all above are not allowed, as they would cause a overflow in the u128 type."); }
+    if n < 76 {
+        return ( GOLDENRATIO.powi(n as i32) / SQRT5 ).round() as u128;
     }
- }
\ No newline at end of file
+
+    let mut x: u128;
+    let mut y: u128 = 1304969544928657;
+    let mut z: u128 = 2111485077978050;
+    let mut i: u128 = 75;
+
+    while i < n {
+        x = y;
+        y = z;
+        z = x + y;
+        i.inc();
+    }
+
+    z
+}
\ No newline at end of file
